# FastApi Odata

![tag](fastapi)
![category](python)

```python
from fastapi import FastAPI, Query
from typing import Optional, List
from pydantic import BaseModel
import uvicorn

app = FastAPI()

# Données fictives
class Item(BaseModel):
    id: int
    name: str
    category: str

fake_db = [
    Item(id=1, name="Item A", category="Cat1"),
    Item(id=2, name="Item B", category="Cat2"),
    Item(id=3, name="Item C", category="Cat1"),
    Item(id=4, name="Item D", category="Cat3"),
]

@app.get("/odata/items")
def get_items(
    $filter: Optional[str] = Query(None, alias="$filter"),
    $select: Optional[str] = Query(None, alias="$select"),
    $orderby: Optional[str] = Query(None, alias="$orderby"),
    $top: Optional[int] = Query(None, alias="$top"),
    $skip: Optional[int] = Query(None, alias="$skip"),
):
    items = fake_db.copy()

    # Filtres simples (ex : category eq 'Cat1')
    if $filter:
        try:
            key, op, value = $filter.split(" ")
            value = value.strip("'")  # Nettoyage des quotes
            if op == "eq":
                items = [item for item in items if getattr(item, key) == value]
        except Exception as e:
            return {"error": "Invalid $filter format", "details": str(e)}

    # Sélection des champs
    if $select:
        fields = $select.split(",")
        items = [{field: getattr(item, field) for field in fields} for item in items]

    # Tri
    if $orderby:
        reverse = False
        if $orderby.endswith(" desc"):
            key = $orderby.replace(" desc", "")
            reverse = True
        else:
            key = $orderby.replace(" asc", "")
        items = sorted(items, key=lambda x: getattr(x, key), reverse=reverse)

    # Pagination
    if $skip:
        items = items[$skip:]
    if $top:
        items = items[:$top]

    return items
```
